﻿using C1.Win.C1Themes;
using System;
using System.Drawing;
using System.Windows.Forms;

namespace FlexGrid_SplitButton
{
    public partial class Form1 : Form
    {
        Random _rnd = new Random();
        public Form1()
        {
            InitializeComponent();

            C1ThemeController.ApplyThemeToControlTree(c1SplitButton1, C1ThemeController.GetThemeByName("MyTheme", false));

            c1FlexGrid1.Cols[1].Name = "editable";
            c1FlexGrid1.Cols[2].Name = "nonEditable";

            c1FlexGrid1.Cols["nonEditable"].AllowEditing = false;

            var style = c1FlexGrid1.Styles.Add("nonEditable");
            style.Border.Color = Color.DarkGray;
            style.Border.Width = 2;
            c1FlexGrid1.Cols["nonEditable"].Style = style;

            style = c1FlexGrid1.Styles.Add("editable");
            style.Border.Color = Color.LightBlue;
            style.Border.Width = 2;
            c1FlexGrid1.Cols["editable"].Style = style;

            style = c1FlexGrid1.Styles.Add("correctVal");
            style.Border.Color = Color.LightGreen;
            style.Border.Width = 2;

            style = c1FlexGrid1.Styles.Add("incorrectVal");
            style.Border.Color = Color.IndianRed;
            style.Border.Width = 2;


            c1FlexGrid1.AfterEdit += C1FlexGrid1_AfterEdit;

            //will run when DrawMode is set to OwnerDraw
            c1FlexGrid1.OwnerDrawCell += C1FlexGrid1_OwnerDrawCell;
        }

        private void C1FlexGrid1_OwnerDrawCell(object sender, C1.Win.C1FlexGrid.OwnerDrawCellEventArgs e)
        {
            if (!e.Measuring && e.Col >= c1FlexGrid1.Cols.Fixed && e.Row >= c1FlexGrid1.Rows.Fixed) 
            {
                var penWidth = 2;
                Color? borderColor = null;
                if (c1FlexGrid1.Cols[e.Col].Name == "editable")
                {
                    borderColor = Color.LightBlue;
                }
                else if(c1FlexGrid1.Cols[e.Col].Name == "nonEditable")
                {
                    borderColor = Color.DarkGray;
                }
                var userData = c1FlexGrid1.GetUserData(e.Row, e.Col)?.ToString() ?? string.Empty;
                if (userData == "correct")
                {
                    borderColor = Color.LightGreen;
                }
                else if(userData == "incorrect")
                {
                    borderColor = Color.IndianRed;
                }

                if(borderColor != null)
                {
                    using (Pen pen = new Pen(borderColor.Value, penWidth))
                    {
                        var rect = e.Bounds;
                        float shrinkAmount = pen.Width / 2;
                        e.Graphics.DrawRectangle(
                            pen,
                            rect.X + shrinkAmount,   // move half a pen-width to the right
                            rect.Y + shrinkAmount,   // move half a pen-width to the down
                            rect.Width - penWidth,   // shrink width with one pen-width
                            rect.Height - penWidth); // shrink height with one pen-width
                    }
                }
            }
        }

        private void C1FlexGrid1_AfterEdit(object sender, C1.Win.C1FlexGrid.RowColEventArgs e)
        {
            if(e.Col == c1FlexGrid1.Cols["editable"].Index)
            {
                //using userData to know if cell contains correct value or not
                if (int.TryParse(c1FlexGrid1[e.Row, e.Col].ToString(), out _))
                {
                    c1FlexGrid1.SetCellStyle(e.Row, e.Col, c1FlexGrid1.Styles["correctVal"]);
                    c1FlexGrid1.SetUserData(e.Row, e.Col, "correct");
                }
                else
                {
                    c1FlexGrid1.SetCellStyle(e.Row, e.Col, c1FlexGrid1.Styles["incorrectVal"]);
                    c1FlexGrid1.SetUserData(e.Row, e.Col, "incorrect");
                }
                if(string.IsNullOrEmpty(c1FlexGrid1[e.Row, e.Col].ToString()))
                {
                    c1FlexGrid1.SetUserData(e.Row, e.Col, string.Empty);
                }
            }
        }

        private void button1_Click(object sender, System.EventArgs e)
        {
            // Get theme by name. 
            var theme = C1ThemeController.GetThemeByName("MyTheme", false);
            if (theme != null)
            {
                // Get the color property at the specified path.
                if (theme.Children.GetItemByPath("C1Input\\C1SplitButton\\Buttons\\Default\\ArrowColor") is ColorProp prop)
                {
                    prop.Value = Color.FromArgb(_rnd.Next(256), _rnd.Next(256), _rnd.Next(256));
                }
                C1ThemeController.ApplyThemeToControlTree(c1SplitButton1, theme);
            }
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            c1FlexGrid1.DrawMode = checkBox1.Checked ? C1.Win.C1FlexGrid.DrawModeEnum.OwnerDraw : C1.Win.C1FlexGrid.DrawModeEnum.Normal;
        }
    }
}
